/* ppc405_0.h - ppc405_0 board header */

/*******************************************************************************
   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE.

   (c) Copyright 2003 Xilinx Inc.
   All rights reserved.
*******************************************************************************/

/*

TODO - Start a new revision history

modification history
--------------------
01a,18feb03,rmm  Ported from VxWorks 5.4 template
01a,09feb04,rmm  CR# 180125. Removed PVR constants
01b,18feb04,rpm  Add PCI support
*/

/*
This file contains I/O addresses and related constants for the
ppc405_0 BSP.
*/

#ifndef _ppc405_0_h_
#define _ppc405_0_h_

/*
 * Include CSP definitions
 */
#include "xparameters.h"

/*
 * No off-board bus interface
 * TODO - Update as required for your system
 */
#define BUS              NONE

/*
 * Exception Vector Prefix Register value.
 * TODO - Update as required for your system
 */
#define EXC_EVPR_VAL     0x00000000

/*
 * Processor core and decrementer clock speed. The name of the parameter
 * constant may be different depending on the toolchain that generated it.
 * We cover all the bases here. Once things settle, this should just reduce
 * down to the first #define (no #ifdefs)
 */
#ifdef XPAR_CPU_PPC405_CORE_CLOCK_FREQ_HZ
#define CORE_CLOCK_FREQ     XPAR_CPU_PPC405_CORE_CLOCK_FREQ_HZ
#else

#ifdef XPAR_CORE_CLOCK_FREQ_HZ
#define CORE_CLOCK_FREQ     XPAR_CORE_CLOCK_FREQ_HZ
#else
/* Default to 300MHz and issue a warning to the user */
#warning "CPU clock frequency needs to be adjusted here - defaults to 300Mhz"
#define CORE_CLOCK_FREQ     300000000
#endif

#endif

/*
 * Number of on-board serial ports that use SIO drivers
 * TODO - Update as required for your system
 */
#define N_SIO_CHANNELS   1

/*
 * Timer tick limits
 * TODO - Update as required
 */
#define SYS_CLK_RATE_MIN  3
#define SYS_CLK_RATE_MAX  5000

#define AUX_CLK_RATE_MIN  (CORE_CLOCK_FREQ / (1 << 21))
#define AUX_CLK_RATE_MAX  (CORE_CLOCK_FREQ / (1 << 9))

#define WDT_RATE_MIN      (CORE_CLOCK_FREQ / (1 << 29))
#define WDT_RATE_MAX      (CORE_CLOCK_FREQ / (1 << 17))

/*
 * The 405 processor core which has a 16KB I-cache and a 8KB D-cache.
 * All 405 processor cores have 32 byte cache lines, and are
 * 2-way set associative.
 */
#define _ICACHE_LINE_NUM_405 256     /* 256 * 32 * 2 = 16KB */
#define _DCACHE_LINE_NUM_405 256     /* 256 * 32 * 2 = 16KB */

/*
 * Define initial settings of cache control registers when there is no
 * MMU support.
 */
#define ppc405_0_ICCR_VAL 0x80000000
#define ppc405_0_DCCR_VAL 0x80000000
#define ppc405_0_DCWR_VAL 0x00000000
#define ppc405_0_SGR_VAL  0x00000000

/*
 * Create a single macro INCLUDE_MMU
 */
#if defined(INCLUDE_MMU_BASIC) || defined(INCLUDE_MMU_FULL)
#define INCLUDE_MMU
#endif

/*
 * Only one can be selected, FULL overrides BASIC
 */
#ifdef INCLUDE_MMU_FULL
#   undef INCLUDE_MMU_BASIC
#endif

/*
 * Local to PCI address translations
 * TODO - Set according to your system
 */
#ifdef INCLUDE_PCI

#   define  PCI_MEM2LOCAL(x) \
    ((void *)((UINT)(x) - PCI_MSTR_MEM_BUS + PCI_MSTR_MEM_LOCAL))

#   define PCI_IO2LOCAL(x) \
    ((void *)((UINT)(x) - PCI_MSTR_IO_BUS + PCI_MSTR_IO_LOCAL))

#   define PCI_MEMIO2LOCAL(x) \
    ((void *)((UINT)(x) - PCI_MSTR_MEMIO_BUS + PCI_MSTR_MEMIO_LOCAL))

#   define LOCAL2PCI_MEM(x) \
    ((void *)((UINT)(x) - PCI_SLV_MEM_LOCAL + PCI_SLV_MEM_BUS))

/*
 * Routines for reading or writing PCI I/O or Memory space. These routines are
 * written in assembler and use byte swapping load/store instructions.
 */
#ifndef PCI_IN_BYTE
#define PCI_IN_BYTE(x)         sysPciInByte(x)
#endif
#ifndef PCI_IN_WORD
#define PCI_IN_WORD(x)         sysPciInWord(x)
#endif
#ifndef PCI_IN_LONG
#define PCI_IN_LONG(x)         sysPciInLong(x)
#endif
#ifndef PCI_OUT_BYTE
#define PCI_OUT_BYTE(x,y)      sysPciOutByte(x,y)
#endif
#ifndef PCI_OUT_WORD
#define PCI_OUT_WORD(x,y)      sysPciOutWord(x,y)
#endif
#ifndef PCI_OUT_LONG
#define PCI_OUT_LONG(x,y)      sysPciOutLong(x,y)
#endif

#endif /* INCLUDE_PCI */

/*
 * Processor version register access
 */
#define CPU_TYPE                  (vxPvrGet())

#endif  /* _ppc405_0_h_ */

