/* $Id: xuartns550_adapter.h,v 1.5 2003/02/14 21:41:35 robertm Exp $ */
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xuartns550_adapter.h
*
* The Xilinx XUartNs550 Adapter component.  This component interfaces between
* the driver component, the WindRiver VxWorks operating system and Serial I/O
* (SIO) interface, and the BSP.  The externally visible functions listed in this
* header file are typically called from the BSP. The xxDevInit() function sets
* up a VxWorks function table, which VxWorks uses to communicate to the adapter.
*
* The adapter takes care to map VxWorks types to Xilinx types where necessary,
* and translates return values as well.
*
* This adapter is intended to be an interface for serial devices only.
*
* A VxWorks BSP contains serial device initialization routines, typically in
* a file called sysSerial.c.  The BSP's sysHwInit() routine calls
* sysSerialHwInit(), which creates an instance of the XUartNs550SioChannel
* structure and calls XUartNs550Adapter_DevInit() to initialize the driver,
* passing it the device ID and interrupt ID of the device. The BSP's
* sysHwInit2() routine typically calls sysSerialHwInit2(), which calls
* XUartNs550Adapter_DevInit2().  This results in the driver connecting and
* enabling interrupts if supported by the device.
*
* <b>Polled Mode</b>
*
* This SIO adapter provides polled send/recv functions in PollSend and PollRecv.
* These functions are typically accessed through the WindRiver debug agent using
* sioPollInput and sioPollOutput functions. It is important for the user to note
* that standard I/O functions like printf/scanf, as well as file I/O functions
* like fputs/fwrite/fread do not work when the UART is in polled mode. VxWorks
* calls the txStartup function of the SIO_CHAN structure when standard or file
* I/O are requested, and this function is intended to work only in interrupt
* mode. If this is a major outage (by the way, most SIO drivers are coded in
* this same manner), the user could modify the txStartup function to go ahead
* and transmit characters when in polled mode rather than return an error. Keep
* in mind that this only fixes standard output (printf) and file output, it does
* not fix standard input (scanf) or file input.  The code snippet that the user
* could add to txStartup (SendStartup) is something like:
*
* <pre>
*
*     if (ChanPtr->ChannelMode == SIO_MODE_POLL)
*     {
*         while ((*ChanPtr->getTxChar)(ChanPtr->getTxArg, &DataChar) != ERROR)
*         {
*             while (PollSend(SioChanPtr, DataChar) == EAGAIN);
*         }
*     }
*     else
*     {
*       ...
* </pre>
*
* @note:
*
* Deviations from normal VxWorks serial drivers:
*    - Since Xilinx intends to support multiple operating systems and
*      processors, we've separated our driver software into two components -
*      the adapter and the driver.  The serial channel structure that is
*      usually available to the BSP is still available, but the information
*      in it is not what one might normally see. That is, driver-specific
*      information like registers and baud rate settings belong in the driver
*      component, so they are not in the adapter's serial channel structure.
*      Some of the driver-specific information is discovered by the driver
*      itself during initialization.  If it is necessary to give the driver
*      more information during initialization, arguments can be added to
*      XUartNs550Adapter_DevInit().
*    - Many VxWorks serial drivers also make their interrupt service routines
*      externally visible.  The purpose of this is so they can be used within
*      the BSP to connect to the interrupt controller and interrupts can be
*      enabled.  We have chosen to bury the connection to the interrupt
*      controller and the enable/disable of interrupts within the adapter.  The
*      adapter makes use of the intConnect and intEnable functions provided by
*      VxWorks.
*
*****************************************************************************/

#ifndef XUARTNS550_ADAPTER_H /* prevent circular inclusions */
#define XUARTNS550_ADAPTER_H /* by using protection macros */

/***************************** Include Files ********************************/

#include "sioLib.h"
#include "xuartns550.h"

/************************** Constant Definitions ****************************/


/**************************** Type Definitions ******************************/

/*
 * Serial channel data structure.  Variables of this type are typically
 * allocated within the BSP.  The BSP then passes a pointer to a channel
 * structure to the adapter's interface functions.
 */
typedef struct
{
    SIO_DRV_FUNCS *pDrvFuncs;    /* set by adapter */

    /* callbacks */
    STATUS (*getTxChar)(void*, ...); /* Function to get a character to send,
                                      * set by adapter */
    void *getTxArg;
    STATUS (*putRcvChar)(void*, ...);/* Function to put a received character,
                                      * set by adapter */
    void *putRcvArg;

    int Options;
    int ChannelMode;

    /*
     * Items below are Xilinx and device specific
     */
    Xuint16 DeviceId;            /* Unique device ID */
    int IntrId;                  /* Interrupt ID */
    XUartNs550 UartNs550;        /* Driver instance */

} XUartNs550SioChannel;


/***************** Macros (Inline Functions) Definitions ********************/


/************************** Function Prototypes *****************************/

/*
 * List the functions visible to the RTOS or the BSP.
 */

XStatus XUartNs550Adapter_DevInit(XUartNs550SioChannel *ChanPtr,
                                  Xuint16 DeviceId, int IntrId);
XStatus XUartNs550Adapter_DevInit2(XUartNs550SioChannel *ChanPtr);


#endif            /* end of protection macro */
