/* romInit.s - generic PPC 4XX ROM initialization module */

/*******************************************************************************
   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE.

   (c) Copyright 2003 Xilinx Inc.
   All rights reserved.
*******************************************************************************/

/*

TODO - Start a new revision history

modification history
--------------------
01a,18feb03,rmm  Ported from VxWorks 5.5 template and ML300
01b,31may06,rmm  CR227241: Fixed to use correct stack address constant
*/

/*
DESCRIPTION 

This module contains the entry code for VxWorks images that start
running from ROM, such as 'bootrom' and 'vxWorks_rom'.
The entry point, romInit(), is the first code executed on power-up.
It performs the minimal setup needed to call
the generic C routine romStart() with parameter BOOT_COLD.

RomInit() typically masks interrupts in the processor, sets the initial
stack pointer (to STACK_ADRS which is defined in configAll.h), and
readies system memory by configuring the DRAM controller if necessary.
Other hardware and device initialization is performed later in the
BSP's sysHwInit() routine.

A second entry point in romInit.s is called romInitWarm(). It is called
by sysToMonitor() in sysLib.c to perform a warm boot.
The warm-start entry point must be written to allow a parameter on
the stack to be passed to romStart().

WARNING:
This code must be Position Independent Code (PIC).  This means that it
should not contain any absolute address references.  If an absolute address
must be used, it must be relocated by the macro ROM_ADRS(x).  This macro
will convert the absolute reference to the appropriate address within
ROM space no matter how the boot code was linked. (For PPC, ROM_ADRS does
not work.  You must subtract _romInit and add ROM_TEXT_ADRS to each
absolute address). (NOTE: ROM_ADRS(x) macro does not work for current
PPC compiler).

This code should not call out to subroutines declared in other modules,
specifically sysLib.o, and sysALib.o.  If an outside module is absolutely
necessary, it can be linked into the system by adding the module 
to the makefile variable BOOT_EXTRA.  If the same module is referenced by
other BSP code, then that module must be added to MACH_EXTRA as well.
Note that some C compilers can generate code with absolute addresses.
Such code should not be called from this module.  If absolute addresses
cannot be avoided, then only ROM resident code can be generated from this
module.  Compressed and uncompressed bootroms or VxWorks images will not
work if absolute addresses are not processed by the macro ROM_ADRS.

*/

#define	_ASMLANGUAGE
        .nolist
#include "vxWorks.h"
#include "sysLib.h"
#include "config.h"
#include "asm.h"
#include "regs.h"

   	/* internals */

	.globl	romInit		/* start of system code */
	.globl	_romInit	/* start of system code */
	.globl	romWarmInit	/* warm start of system code */
	.globl	_romWarmInit	/* warm start of system code */

	/* externals */

        .extern _romStart
   
	.text
	.align 4

/*******************************************************************************
*
* romInit - entry point for VxWorks in ROM.
*

* romInit
*     (
*     int startType	/@ only used by 2nd entry point @/
*     )

*/

romInit:
_romInit:
   bl      cold
   nop

   /* warm reboot entry point */

romInitWarm:
_romInitWarm:
   bl      warm

   /* copyright notice appears at beginning of ROM (in TEXT segment) */

   .ascii   "Copyright 1984-1998 Wind River Systems, Inc.  "
   .ascii   "Copyright 2003 Xilinx, Inc.   "
   .align 4
   
   /* on cold-boots, delay to allow time for SDRAM to initialize itself */
cold:
   li      r5,BOOT_COLD            /* Save start type in r5 as BOOT_COLD */
   b       start
			
warm:
   or      r5,r3,r3                /* Save start type in r5 */
   b       start

start:

   /*
    * Disable everything except machine checks
    */
   addi    r4,r0,0                 /* r4 = 0x00000000    */
   mtmsr   r4                      /* msr = r4 */
   isync

   /*
    * Zero out registers
    */
   mtspr   TBL, r4                 /* Clear out the time-base regs  */
   mtspr   TBU, r4                 /* Clear out the time-base regs  */
   mtspr   ICCR,r4                 /* turn off I-cache              */
   mtspr   DCCR,r4                 /* turn off D-cache              */
   mtspr   DCWR,r4                 /* all memory regions write-back */
   mtspr   ESR, r4                 /* clear error status            */
   mtspr   TCR, r4                 /* disable all timers            */
   mtspr   PIT, r4                 /* clear the PIT timer           */   

   /*
    * Set registers to 0xFFFFFFFF
    */
   addi    r4,r0,-1                /* r4 = 0xFFFFFFFF               */
   mtspr   TSR, r4                 /* clear timer status            */
   mtspr   DBSR, r4                /* clear debug status reg        */   

   /*
    * Invalidate the entire instruction cache. This can be done
    * with a single iccci instruction in the 405 processor core.
    */
   iccci   r0, r0

   /*
    * Invalidate the entire data cache.
    * The 405 processor core has 128 congruence classes.
    * Each cache line in the 405 processor is 32 bytes.
    */
   li      r3, 0                      /* clear r3 */
   li      r4, _DCACHE_LINE_NUM_405   /* load number of cache lines */
   mtctr   r4

rom405Dcci:
   dccci   r0, r3
   addi    r3, r3, _CACHE_ALIGN_SIZE  /* bump to next line */
   bdnz    rom405Dcci                 /* go to invalidate */

   /*
    * Set W attribute
    */
   lis     r4,HIADJ(ppc405_0_DCWR_VAL)
   addi    r4,r4,LO(ppc405_0_DCWR_VAL)
   mtspr   DCWR,r4

   /*
    * Turn on the instruction cache.
    */
#ifdef USER_I_CACHE_ENABLE
   lis     r4,HIADJ(ppc405_0_ICCR_VAL)
   addi    r4,r4,LO(ppc405_0_ICCR_VAL)
   mtspr   ICCR,r4
   isync
#endif

   /* 
    * Initialize the stack pointer 
    */
   LOADPTR (sp, STACK_ADRS)
   addi    sp, sp, -FRAMEBASESZ

   /* calculate C entry point for romStart() = routine - entry point + ROM base */

   lis     r6,HIADJ(romStart)      /* r6 = romStart */
   addi    r6, r6, LO(romStart)

   lis     r7, HIADJ(romInit)      /* r7 = romInit */
   addi    r7, r7, LO(romInit)

   lis     r8, HIADJ(ROM_TEXT_ADRS) /* r8 = ROM_TEXT_ADRS */
   addi    r8, r8, LO(ROM_TEXT_ADRS)

   subf    r6, r7, r6              /* r6 = r6 - r7 */
   add     r6, r6, r8              /* r6 = r6 + r8 */
   mtlr    r6                      /* link register = C entry point */

   or      r3, r5, r5              /* r3 = startType */
   addi    sp, sp, -FRAMEBASESZ    /* get frame stack */

   blr                             /* call romStart(starType), does not return */
   b       .                       /* incase romStart() returns */


