/* sysALib.s - 405 Core system-dependent assembly routines */
   
/*******************************************************************************
   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE.

   (c) Copyright 2003-2006 Xilinx Inc.
   All rights reserved.
*******************************************************************************/
   
/*

TODO - Start a new revision history

modification history
--------------------
01a,18feb03,rmm  Ported to VxWorks 5.5 template and ML300
01b,7oct05,rmm   CR212451: Added workaround for CPU errata 213.
01c,30may06,rmm  CR230411: Implemented faster cache flush routine
*/
/*
DESCRIPTION
This module contains the entry code, sysInit(), for VxWorks images that start
running from RAM, such as 'vxWorks'. These images are loaded into memory  
by some external program (e.g., a boot ROM) and then started. 
The routine sysInit() must come first in the text segment. Its job is to  
perform the minimal setup needed to call the generic C routine usrInit() with 
parameter BOOT_COLD. 

The routine sysInit() typically masks interrupts in the processor, sets the
initial stack pointer to _sysInit, then jumps to usrInit.
Most other hardware and device initialization is performed later by
sysHwInit().

*/
#define _ASMLANGUAGE
#include "vxWorks.h"
#include "sysLib.h"
#include "config.h"
#include "asm.h"
#include "regs.h"
#include "cacheLib.h"
   
   /* globals */
   .globl  FUNC(_sysInit)          /* start of system code */
   FUNC_DECL(.globl, _sysInit)
   
   .globl  FUNC(sysMemProbeSup)
   FUNC_DECL(.globl, sysMemProbeSup)
   
   .globl  sysSpin
   FUNC_DECL(.globl, sysSpin)
   
   .globl  sysInByte
   FUNC_DECL(.globl, sysInByte)
   
   .globl  sysOutByte
   FUNC_DECL(.globl, sysOutByte)
   
   .globl  sysInWord
   FUNC_DECL(.globl, sysInWord)
   
   .globl  sysOutWord
   FUNC_DECL(.globl, sysOutWord)

   .globl  sysInLong
   FUNC_DECL(.globl, sysInLong)
   
   .globl  sysOutLong
   FUNC_DECL(.globl, sysOutLong)

   .globl  sysPciInByte
   FUNC_DECL(.globl, sysInPciByte)
   
   .globl  sysPciOutByte
   FUNC_DECL(.globl, sysPciOutByte)
   
   .globl  sysPciInWord
   FUNC_DECL(.globl, sysPciInWord)
   
   .globl  sysPciOutWord
   FUNC_DECL(.globl, sysPciOutWord)

   .globl  sysPciInLong
   FUNC_DECL(.globl, sysPciInLong)
   
   .globl  sysPciOutLong
   FUNC_DECL(.globl, sysPciOutLong)

   .globl  sysDcrPlbbearGet
   FUNC_DECL(.globl, sysDcrPlbbearGet)
   
   .globl sysDcrPlbbesrGet
   FUNC_DECL(.globl, sysDcrPlbbesrGet)

   .globl  sysDcrPlbbesrClear
   FUNC_DECL(.globl, sysDcrPlbbesrClear)

#if (defined INCLUDE_ALTERNATE_CACHE_FLUSH) && (defined INCLUDE_CACHE_SUPPORT)
   .globl  cacheArchFlush_ppc405_revD_and_up
   FUNC_DECL(.globl, cacheArchFlush_ppc405_revD_and_up)
#endif
   
   /* externals */
   .extern FUNC(usrInit)

#if (defined INCLUDE_ALTERNATE_CACHE_FLUSH) && (defined INCLUDE_CACHE_SUPPORT)
   .extern FUNC(cacheErrnoSet)
#endif

   .text

/*******************************************************************************
*
* sysInit - start after boot
*
* This routine is the system start-up entry point for VxWorks in RAM, the
* first code executed after booting.  It disables interrupts, sets up the
* stack, and jumps to the C routine usrInit() in usrConfig.c.
*
* The initial stack is set to grow down from the address of sysInit().  This
* stack is used only by usrInit() and is never used again.  Memory for the
* stack must be accounted for when determining the system load address.
*
* NOTE: This routine should not be called by the user.
*
* RETURNS: N/A

* sysInit (void)        /@ THIS IS NOT A CALLABLE ROUTINE @/

*/
_sysInit:
   /*
    * Disable everything except machine checks
    */
   addi    r4,r0,0                 /* r4 = 0x00000000    */
   mtmsr   r4                      /* msr = r4 */
   isync

   /*
    * Zero out registers
    */
   mtspr   TBL, r4                 /* Clear out the time-base regs  */
   mtspr   TBU, r4                 /* Clear out the time-base regs  */
   mtspr   ICCR,r4                 /* turn off I-cache              */
   mtspr   DCCR,r4                 /* turn off D-cache              */
   mtspr   DCWR,r4                 /* all memory regions write-back */
   mtspr   ESR, r4                 /* clear error status            */
   mtspr   TCR, r4                 /* disable all timers            */
   mtspr   PIT, r4                 /* clear the PIT timer           */   

   /*
    * Set registers to 0xFFFFFFFF
    */
   addi    r4,r0,-1                /* r4 = 0xFFFFFFFF               */
   mtspr   TSR, r4                 /* clear timer status            */
   mtspr   DBSR, r4                /* clear debug status reg        */   

   /*
    * CPU errata 213: Set bits 1 and 3 of CCR0 to prevent incorrect data from
    * being flushed out of the data cache under certain conditions on the PLB.
    */
   mfccr0  r0
   oris    r0,r0,0x5000
   mtccr0  r0

   /*
    * Invalidate the entire instruction cache. This can be done
    * with a single iccci instruction in the 405 processor core.
    */
   iccci   r0, r0

   /*
    * Invalidate the entire data cache.
    * The 405 processor core has 128 congruence classes.
    * Each cache line in the 405 processor is 32 bytes.
    */
   li      r3, 0                      /* clear r3 */
   li      r4, _DCACHE_LINE_NUM_405   /* load number of cache lines */
   mtctr   r4

rom405Dcci:
   dccci   r0, r3
   addi    r3, r3, _CACHE_ALIGN_SIZE  /* bump to next line */
   bdnz    rom405Dcci                 /* go to invalidate */

   /*
    * Set W attribute
    */
   lis     r4,HIADJ(ppc405_0_DCWR_VAL)
   addi    r4,r4,LO(ppc405_0_DCWR_VAL)
   mtspr   DCWR,r4

   /*
    * Turn on the instruction cache.
    */
#if (defined INCLUDE_CACHE_SUPPORT) && (defined USER_I_CACHE_ENABLE)
   lis     r4,HIADJ(ppc405_0_ICCR_VAL)
   addi    r4,r4,LO(ppc405_0_ICCR_VAL)
   mtspr   ICCR,r4
   isync
#endif   

   /* 
    * Initialize the stack pointer 
    */
   LOADPTR (sp, RAM_LOW_ADRS)
   addi    sp, sp, -FRAMEBASESZ
   
      /* 
       * Clear 1st byte of boot line
       */
#ifdef SYSINIT_CLEAR_BOOT_LINE
   LOADPTR (r4, SYSINIT_CLEAR_BOOT_LINE_ADRS)
   li      r3,0
   stb     r3,0(r4)
#endif

   /* Jump to usrInit */
   li      r3, BOOT_COLD  /* set start type arg */
   bl      FUNC(usrInit)

   /* usrInit should not return, but just in case */
   bl      sysSpin

      
/*******************************************************************************
*
* sysPciInByte - read one byte from an I/O address in PCI space
*
* This routine reads a single byte from a specified I/O address.
*   
* ARGUMENTS:
*    address (r3)
*   
* RETURNS:
*    r3 - byte from address.
*/

/*******************************************************************************
*
* sysInByte - read one byte from an I/O address
*
* This routine reads a single byte from a specified I/O address.
*   
* ARGUMENTS:
*    address (r3)
*   
* RETURNS:
*    r3 - byte from address.
*/

sysPciInByte:
sysInByte: 
   eieio                           /* Synchronize instruction pipeline */
   lbz     r3,0(r3)                /* Load from address */
   blr                             /* Return */


/*******************************************************************************
*
* sysPciOutByte - write one byte to an I/O address in PCI space
*
* This routine writes a single byte to a specified I/O address.
*
* ARGUMENTS:
*    address (r3)
*    data (r4)
*   
* RETURNS: N/A
*/

/*******************************************************************************
*
* sysOutByte - write one byte to an I/O address
*
* This routine writes a single byte to a specified I/O address.
*
* ARGUMENTS:
*    address (r3)
*    data (r4)
*   
* RETURNS: N/A
*/

sysPciOutByte:
sysOutByte:
   stb     r4,0(r3)                /* Store data */
   eieio                           /* Enforce in-order I/O */
   blr                             /* Return */


/*****************************************************************************
*
* sysInWord - reads a half word from an I/O address.
*
* This function reads a 16 bit half word from a specified I/O address.
*
* ARGUMENTS:
*    address (r3)
*
* RETURNS:
*    r3 - word from address.
*/

sysInWord:
    eieio                   /* Sync I/O operation */
    lhzx    r3,r0,r3        /* Read word from an address */
    bclr    20,0            /* Return to caller */

   
/******************************************************************************
*
* sysOutWord - write a half word to an I/O address.
*
* This function write a 16 bit half word to a specified io address.
*
* ARGUMENTS:
*    address (r3)
*    data (r4)
*
* RETURNS: N/A
*/

sysOutWord:
    sthx    r4,r0,r3        /* Write a word to an address */
    eieio                   /* Sync I/O operation */
    bclr    20,0            /* Return to caller */


/*****************************************************************************
*
* sysInLong - reads a word from an io address.
*
* This function reads a 32 bit word from a specified io address.
*
* ARGUMENTS:
*    address (r3)
*
* RETURNS:
*    r3 - long word from address.
*/

sysInLong:
        eieio                   /* Sync I/O operation */
        lwzx    r3,r0,r3        /* Read word from an address */
        bclr    20,0            /* Return to caller */


/******************************************************************************
*
* sysOutLong - writes a word to an io address.
*
* This function writes a 32 bit word to a specified io address.
*
* ARGUMENTS:
*    address (r3)
*    data (r4)
*
* RETURNS: N/A
*/

sysOutLong:
        stwx    r4,r0,r3        /* Write a word to an address */
        eieio                   /* Sync I/O operation */
        bclr    20,0            /* Return to caller */


/*****************************************************************************
*
* sysPciInWord - reads a word (16-bit big-endian) from PCI Config Space.
*
* This function reads a word from a specified PCI Config Space (little-endian)
* address.  It uses the load halfword byte-reversed instruction.
*
* ARGUMENTS:
*    address (r3)
*
* RETURNS:
*    r3 - word from address.   
*/

sysPciInWord:
   eieio                   /* Sync I/O operation */
   lhbrx   r3,r0,r3        /* Read and swap from PCI space */
   bclr    20,0            /* Return to caller */

   
/******************************************************************************
*
* sysPciOutWord - writes a word (16-bit big-endian) to PCI Config Space.
*
* This function writes a word to a specified PCI Config Space (little-endian)
* address.  It uses the store halfword byte-reversed instruction.
*
* ARGUMENTS:
*    address (r3) 
*    data (r4) 
*
* RETURNS: N/A
*/

sysPciOutWord:
        sthbrx  r4,r0,r3        /* Write with swap to PCI space */
        eieio                   /* Sync I/O operation */
        bclr    20,0            /* Return to caller */


/*****************************************************************************
*
* sysPciInLong - reads a long (32-bit big-endian) from PCI Config Space.
*
* This function reads a long from a specified PCI Config Space (little-endian)
* address.  It uses the load word byte-reversed instruction.
*
* ARGUMENTS:
*    address (r3)
*
* RETURNS:
*    r3 - word from address.   
*/

sysPciInLong:
        eieio                   /* Sync I/O operation */
        lwbrx   r3,r0,r3        /* Read and swap from PCI space */
        bclr    20,0            /* Return to caller */

      
/******************************************************************************
*
* sysPciOutLong - writes a long (32-bit big-endian) to PCI Config Space.
*
* This function writes a long to a specified PCI Config Space (little-endian)
* address.  It uses the store word byte-reversed instruction.
*
* ARGUMENTS:
*    address (r3)
*    data (r4) 
*
* RETURNS: N/A
*/

sysPciOutLong:
        stwbrx  r4,r0,r3        /* Write big-endian long to little-endian */
        eieio                   /* Sync I/O operation */
        bclr    20,0            /* Return to caller */


/*****************************************************************************
*
* sysSpin - performs a branch hang
*
*
* ARGUMENTS:
*       none
*
* RETURNS:
*       does not return
*/
sysSpin:
   b       sysSpin

/*****************************************************************************
*
* sysDcrPlbbearGet - retrieve PLB bus error address 
*
* There can be two possible sources for this information. The PLB2OPB bridge
* or the PLB arbiter. The bridge will store an address if the access was to
* an invalid address in OPB space. The arbiter will store an address if the
* access was to an invalid address in PLB space.
*
* If both devices indicate an error, then the bridge's error address is given
* priority and returned.
*
* ARGUMENTS: 
*    none
*
* RETURNS: 
*    Value of PEAR register (PLB arbiter), or BEAR register (PLB2OPB bridge),
*    or 0 if neither component was compiled into the system
*/
sysDcrPlbbearGet:

   /* No PLB support in the system, return 0 */
   LOADPTR(r3,0)                /* default return value */
   blr
   

/*****************************************************************************
*
* sysDcrPlbbesrGet - wrapper used to get the contents of the PLB BESR register
*
* There can be two possible sources for this information. The PLB2OPB bridge
* or the PLB arbiter. The bridge will store a status if the access was to
* an invalid address in OPB space. The arbiter will store a status if the
* access was to an invalid address in PLB space.
*
* If both devices indicate an error, then the arbiter's status is given
* priority and returned.
*
* ARGUMENTS: 
*    none
*
* RETURNS: 
*    Value of PESR MERR register (PLB arbiter), or BESR MERR register (PLB2OPB
*    bridge), or 0 if neither component was compiled into the system or no 
*    error was indicated by any of the present devices.
*/
sysDcrPlbbesrGet:

   /* No PLB support in the system, return 0 */
   LOADPTR(r3,0)                /* default return value */
   blr                          /* return     */


/*****************************************************************************
*
* sysDcrPlbbesrClear - clear PLB bus error status
*
* Both PLB2OPB bridge and PLB arbiter status is cleared.
*
* ARGUMENTS:
*       r3 - clear argument
*
* RETURNS:
*       nothing
*/
sysDcrPlbbesrClear:  
   blr                         /* return */

/******************************************************************************
*
* cacheArchFlush_ppc405_revD_and_up - flush entries in a PowerPC cache
*
* This routine replaces cacheLib function cacheArchFlush. The cacheLib version
* of this function must cope with errata 37 of the PPC405 RevC core. Virtex
* PPCs are RevD or higher. This implementation will be noticeably faster.
*
* This implementation is enabled when INCLUDE_ALTERNATE_CACHE_FLUSH is defined
* in config.h, otherwise the VxWorks library routine is used.
*
* RETURNS: OK, or ERROR if the cache type is invalid or the cache control
* is not supported.
*
* STATUS cacheArchFlush_ppc405_revD_and_up
*     (
*     CACHE_TYPE  cache,          /@ cache to flush @/
*     void *      address,        /@ virtual address @/
*     size_t      bytes           /@ number of bytes to flush @/
*     )
*/
#if (defined INCLUDE_ALTERNATE_CACHE_FLUSH) && (defined INCLUDE_CACHE_SUPPORT)
cacheArchFlush_ppc405_revD_and_up:
   cmpwi   r3,_DATA_CACHE
   beq     startFluDat

   mfspr   r0,LR                       /* No flushing of icache allowed here */
   stw     r0,4(sp)                    /* Save some space and call error logging */
   stwu    sp,-16(sp)                  /* func and return */
   bl      cacheErrnoSet
   lwz     r0,20(sp)
   addi    sp,sp,16
   mtspr   LR,r0
   blr
   
startFluDat:   
   LOADPTR(r3,ppc405CACHE_ALIGN_SIZE)  /* Get the cache line size */
   lwz     r3,0(r3)
   add     r5,r5,r4                    /* Calculate last address */
   clrrwi  r4,r4,5                     /* Round address to 32 byte boundary */

fluDat:
   dcbst   0,r4
   add     r4,r4,r3
   cmpl    0,r4,r5
   blt     fluDat
   sync

   li      r3,OK
   blr
#endif
   
