/* sysCache.c - PPC405 system cache and MMU setup */

/*******************************************************************************
   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE.

   (c) Copyright 2003 Xilinx Inc.
   All rights reserved.
*******************************************************************************/

/* Copyright 1984-1999 Wind River Systems, Inc. */

/*

TODO - Start a new revision history
  
modification history
--------------------
01a,18feb03,rmm  Ported from ML300 version 01b
01b,08may06,wsy  Add support for vxWorks 6.1 EDR PM.

*/

/*
DESCRIPTION
This source code file defines cache and MMU related data structures used during
system initialization. 

TODO - Change this file as required to suite your system requirements.

SEE ALSO:
.pG "Configuration"
*/

/* includes */

#include "vxWorks.h"
#include "memLib.h"
#include "cacheLib.h"
#include "config.h"
#include "arch/ppc/mmu405Lib.h"
#include "arch/ppc/vxPpcLib.h"

/*
 * The following constants specify cache attributes for the 405 Core. They
 * are used by the architecture specific cache library.
 */

/* no. of lines in ICACHE */
const int ppc405ICACHE_LINE_NUM = _ICACHE_LINE_NUM_405;

/* no. of lines in DCACHE */
const int ppc405DCACHE_LINE_NUM = _DCACHE_LINE_NUM_405;

/* cache line size */
const int ppc405CACHE_ALIGN_SIZE = _CACHE_ALIGN_SIZE;     

/*
 * sysPhysMemDesc[] is used to initialize the Page Table Entry (PTE) array
 * used by the MMU to translate addresses with single page (4k) granularity.
 *
 * Address translations for local RAM, memory mapped PCI bus, memory mapped
 * IO space and local PROM/FLASH are set here.
 *
 * PTEs are held in a 2-level page table. There is one Level 1 page table
 * and several Level 2 page tables. The size of the Level 1 table is 4K
 * and the size of each Level 2 page table is 8K. Each Level 2 table can
 * map upto 4MB of contiguous memory space.
 * 
 */
#if defined(INCLUDE_MMU_BASIC) || defined(INCLUDE_MMU_FULL)
PHYS_MEM_DESC sysPhysMemDesc [] =
{
    /* Main memory. VxWorks images & heap space */
    {
        (void *) LOCAL_MEM_LOCAL_ADRS,
        (void *) LOCAL_MEM_LOCAL_ADRS,
        LOCAL_MEM_SIZE - USER_RESERVED_MEM,
        VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
        VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE
    },
    /* Reserved memory */
    {
        (void *) LOCAL_MEM_LOCAL_ADRS + LOCAL_MEM_SIZE - USER_RESERVED_MEM,
        (void *) LOCAL_MEM_LOCAL_ADRS + LOCAL_MEM_SIZE - USER_RESERVED_MEM,
        USER_RESERVED_MEM,
        VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
        VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },
#ifdef INCLUDE_EDR_PM
    /* PM Reserved memory */
    {
        (void *) LOCAL_MEM_LOCAL_ADRS + LOCAL_MEM_SIZE - USER_RESERVED_MEM - PM_RESERVED_MEM,
        (void *) LOCAL_MEM_LOCAL_ADRS + LOCAL_MEM_SIZE - USER_RESERVED_MEM - PM_RESERVED_MEM,
        PM_RESERVED_MEM,
        VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
        VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },
#endif

    /* TODO - add other regions specific to the system */
};

int sysPhysMemDescNumEnt = NELEMENTS (sysPhysMemDesc);

#endif   /* ifdef INCLUDE_MMU_FULL or INCLUDE_MMU_BASIC */



/* externals */


/* These are set in sysHwInit() */
IMPORT UINT32   ppc405IccrVal;    /* cachability state for instruction */
IMPORT UINT32   ppc405DccrVal;    /* cachability state for data */
