/* sysLib.c - ppc405_0 system-dependent library */

/*******************************************************************************
   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE.

   (c) Copyright 2003-2006 Xilinx Inc.
   All rights reserved.
*******************************************************************************/

/* Copyright 1984-1999 Wind River Systems, Inc. */

/*

TODO - Start a new revision history

modification history
--------------------
01a,18feb03,rmm  Ported from ML300 version 01b
01b,09feb04,rmm  CR 180125. Removed PVR check in sysModel()
                 CR 180420. Removed call to VM_ENABLE() in sysToMonitor().
01c,08may06,wsy  Add support for vxWorks 6.1 EDR PM
01d,30may06,rmm  CR 230411. Implemented faster data cache flush function.
*/

/*
DESCRIPTION
This library provides board-specific routines. Drivers from Wind River are
typically #include'd, while drivers from Xilinx are compiled separately.

TODO - Add your own specific board routines here as needed.

INCLUDE FILES: sysLib.h sysLibExtra.h

SEE ALSO:
.pG "Configuration"
*/

/* includes */

#include "vxWorks.h"
#include "memLib.h"
#include "cacheLib.h"
#include "sysLib.h"
#include "sysLibExtra.h"
#include "config.h"
#include "string.h"
#include "stdio.h"
#include "intLib.h"
#include "logLib.h"
#include "taskLib.h"
#include "vxLib.h"
#include "tyLib.h"
#include "vmLib.h"
#include "arch/ppc/mmu405Lib.h"
#include "arch/ppc/vxPpcLib.h"

#include "xbasic_types.h"
#include "xio.h"

/* defines */

/* globals */

int   sysBus;                       /* system bus type */
int   sysCpu      = PPC405;         /* system CPU type (PPC405 Core) */
char *sysBootLine = BOOT_LINE_ADRS; /* address of boot line */
char *sysExcMsg   = EXC_MSG_ADRS;   /* catastrophic message area */
int   sysProcNum;                   /* processor number of this CPU */
int   sysFlags;                     /* boot flags */
char  sysBootHost [BOOT_FIELD_LEN]; /* name of host from which we booted */
char  sysBootFile [BOOT_FIELD_LEN]; /* name of file from which we booted */
int   sysTimerClkFreq;              /* Decrementer frequency. Usually this value
                                       is a function of the bus speed */
int   sysXInitErrorCtr = 0;         /* Number of errors seen during boot from
                                       Xilinx device drivers */

/*
 * If boot error log is enabled, then whenever a CSP init function fails
 * the file and location is logged. Otherwise a counter is incremented
 */
#ifdef INCLUDE_CSP_BOOT_ERROR_LOG
#  define BOOT_LOG_ERROR() sysCspBootErrorLog(__LINE__,__FILE__)
   static void  sysCspBootErrorLog(int line, char* file);
   static char* sysCspErrorFile[MAX_CSP_BOOT_ERROR_RECORDS];
   static int   sysCspErrorLine[MAX_CSP_BOOT_ERROR_RECORDS];
#else
#  define BOOT_LOG_ERROR() sysXInitErrorCtr++
#endif

/* locals */

/* forward declarations */

char*       sysPhysMemTop (void);
int         sysGetBusSpd (void);
static void sysAssert(char* file, int line);

/*
 * BSP DRIVERS
 *
 * Note that these are not the only drivers in the system. Other drivers are
 * contained in the CSP. These drivers are compiled as separate units
 * and not #include'd here.
 */

#ifdef INCLUDE_PCI
#   include "pci/pciConfigLib.c"
#   include "pci/pciIntLib.c"
#   ifdef INCLUDE_PCI_AUTOCONF
#       include "pci/pciAutoConfigLib.c"
#       include "sysBusPci.c"
#   endif
#endif

#include "mem/nullNvRam.c"
#include "ppc405Timer.c"
#include "sysCache.c"
#include "sysPlbOpb.c"
#include "sysInterrupt.c"

#ifdef INCLUDE_XSYSACE
#   include "sysSystemAce.c"
#endif

#ifdef INCLUDE_XGPIO
#   include "sysGpio.c"
#endif

#ifdef INCLUDE_SIO
#   include "sysSerial.c"
#endif

#ifdef INCLUDE_NETWORK
#   include "sysNet.c"
#endif

/******************************************************************************
*
* sysModel - return the model name of the CPU board
*
* This routine returns the model name of the CPU board. The returned string
* depends on the board model and CPU version being used.
*
* RETURNS: A pointer to the string.
*/

char * sysModel (void)
{
/*    return ("ppc405_0 VirtexII Pro PPC405");*/
    return ("Xilinx Virtex-4 FX PPC405");
}


/*******************************************************************************
*
* sysBspRev - return the BSP version and revision number
*
* This routine returns a pointer to a BSP version and revision number, for
* example, 1.1/0. BSP_REV is concatenated to BSP_VERSION and returned.
*
* RETURNS: A pointer to the BSP version/revision string.
*/

char * sysBspRev (void)
{
    return (BSP_VERSION BSP_REV);
}

/*******************************************************************************
*
* sysHwInit - initialize the system hardware
*
* This routine initializes various features of the board. It is the first
* board-specific C code executed, and runs with interrupts masked in the
* processor. This routine resets all devices to a quiescent state, typically
* by calling driver initialization routines.
*
* NOTE: Because this routine will be called from sysToMonitor, it must
* shutdown all potential DMA master devices.  If a device is doing DMA
* at reboot time, the DMA could interfere with the reboot. For devices
* on non-local busses, this is easy if the bus reset or sysFail line can
* be asserted.
*
* NOTE: This routine should not be called directly by the user application.
*
* RETURNS: N/A
*/

void sysHwInit (void)
{
    /* Set the 405 Exception Vector Prefix Register */
    vxEvprSet(EXC_EVPR_VAL);

    /* Setup PLB/OPB bridges & arbiters */
    sysPlbOpbInit();

    /* Setup GPIO if available */
#ifdef INCLUDE_XGPIO
    sysGpioInit();
#endif

    /* Setup CSP assert visual notification */
#ifndef NDEBUG
    sysAssert((char*)XNULL, 0);
#endif

    /* Setup interrupt control */
    sysIntInit();

#ifdef INCLUDE_XSYSACE
    sysSystemAceInit();
#endif

    /*
     * Setup the system timer frequency for the decrementer timer driver
     */
    sysTimerClkFreq = CORE_CLOCK_FREQ;

#ifdef INCLUDE_NETWORK
    sysNetHwInit();     /* network interface */
#endif

#ifdef INCLUDE_SIO
    sysSerialHwInit();  /* serial devices */
#endif

#ifdef INCLUDE_PCI
    /*
     * Initialize the PCI configuration library, and the PCI bridge
     */
    pciIntLibInit();
    pciConfigLibInit(PCI_MECHANISM_1, (ULONG)PCI_CFGADDR,
                     (ULONG)PCI_CFGDATA, 0);
    sysPciHostBridgeInit();

#   ifdef INCLUDE_PCI_AUTOCONF
        /* Perform auto-configuration of devices on the PCI bus */
        sysPciAutoConfig();
#   endif   /* INCLUDE_PCI_AUTOCONF */

#endif

#ifdef INCLUDE_FLASH
    /* TODO - any Flash ROM hardware setup/reset */
#endif

    /*
     * ppc405IccrVal and ppc405DccrVal define respectively the cachability
     * state of the total effective adress space at a 128 MB granularity
     * for instruction and data. See sysCache.c & ppc405_0.h
     */
#ifdef  INCLUDE_CACHE_SUPPORT
    ppc405IccrVal = ppc405_0_ICCR_VAL;
    ppc405DccrVal = ppc405_0_DCCR_VAL;

#ifdef INCLUDE_ALTERNATE_CACHE_FLUSH
    cacheLib.flushRtn = cacheArchFlush_ppc405_revD_and_up;
#endif
    
#endif

}

/*******************************************************************************
*
* sysHwInit2 - initialize additional system hardware
*
* This routine connects system interrupt vectors and configures any
* required features not configured by sysHwInit(). It is called from usrRoot()
* in usrConfig.c after the multitasking kernel has started.
*
* RETURNS: N/A
*/

void sysHwInit2 (void)
{
    static BOOL configured = FALSE;

    if (!configured)
    {
        /* connect system clock interrupt : excIntConnectTimer uses malloc */
        excIntConnectTimer ((VOIDFUNCPTR *) _EXC_OFF_PIT,
                            (VOIDFUNCPTR) sysClkInt);

        /* connect auxiliary clock interrupt */
        excIntConnectTimer ((VOIDFUNCPTR *) _EXC_OFF_FIT,
                            (VOIDFUNCPTR) sysAuxClkInt);

        sysPlbOpbInit2();       /* PLB/OPB bridges & arbiters */

#ifdef INCLUDE_NETWORK
        sysNetHwInit2 ();       /* network interface */
#endif

        /* connect serial interrupts */
#ifdef INCLUDE_SIO
        sysSerialHwInit2();
#endif

        configured = TRUE;
    }
}



/*******************************************************************************
*
* sysPhysMemTop - get the address of the top of physical memory
*
* This routine returns the address of the first missing byte of memory,
* which indicates the top of memory.
*
* Normally, the user specifies the amount of physical memory with the
* macro LOCAL_MEM_SIZE in config.h.  BSPs that support run-time
* memory sizing do so only if the macro LOCAL_MEM_AUTOSIZE is defined.
* If not defined, then LOCAL_MEM_SIZE is assumed to be, and must be, the
* true size of physical memory.
*
* NOTE: Do no adjust LOCAL_MEM_SIZE to reserve memory for application
* use.  See sysMemTop() for more information on reserving memory.
*
* RETURNS: The address of the top of physical memory.
*
* SEE ALSO: sysMemTop()
*/
char * sysPhysMemTop (void)
{
    static char * physTop = NULL;

    if (physTop == NULL)
    {
#ifdef LOCAL_MEM_AUTOSIZE
# error "Dynamic memory sizing not supported"
#else
        /* Don't do autosizing, if size is given */
        physTop = (char *)(LOCAL_MEM_LOCAL_ADRS + LOCAL_MEM_SIZE);
#endif /* LOCAL_MEM_AUTOSIZE */
    }
    return physTop;
}


/*******************************************************************************
*
* sysMemTop - get the address of the top of VxWorks memory
*
* This routine returns a pointer to the first byte of memory not
* controlled or used by VxWorks.
*
* The user can reserve memory space by defining the macro USER_RESERVED_MEM
* in config.h.  This routine returns the address of the reserved memory
* area.  The value of USER_RESERVED_MEM is in bytes.
*
* RETURNS: The address of the top of VxWorks memory.
*/
char * sysMemTop (void)
{
    static char * memTop = NULL;

    if (memTop == NULL)
    {
        memTop = sysPhysMemTop () - USER_RESERVED_MEM;

#ifdef INCLUDE_EDR_PM
        /* account for ED&R persistent memory */

        memTop = memTop - PM_RESERVED_MEM;
#endif
    }

    return memTop;
}

/*******************************************************************************
*
* sysToMonitor - transfer control to the ROM monitor
*
* This routine will reboot the system to either the System ACE default
* configuration address or the address set by sysSystemAceSetRebootAddr().
* Normally, it is called only by reboot()--which services ^X--and by bus
* errors at interrupt level. However, in some circumstances, the user may
* wish to introduce a <startType> to enable special boot ROM facilities.
*
* RETURNS: Does not return.
*/
STATUS sysToMonitor
    (
    int startType    /* parameter passed to ROM to tell it how to boot */
    )
{
    intLock();            /* disable interrupts */

    cacheDisable (INSTRUCTION_CACHE);   /* Disable caches */
    cacheDisable (DATA_CACHE);

    sysHwInit ();             /* disable all sub systems to a quiet state */
    vxMsrSet (0);             /* Clear the MSR */

    /* decide how to reboot */
#if defined INCLUDE_STANDARD_WARMBOOT
    (*(FUNCPTR)(ROM_WARM_ADRS))(startType);
#elif defined INCLUDE_XSYSACE
    sysSystemAceJtagReboot();
#else
    while(1);
#endif

    return (OK);              /* in case we continue from ROM monitor */
}

/******************************************************************************
*
* sysProcNumGet - get the processor number
*
* This routine returns the processor number for the CPU board, which is
* set with sysProcNumSet().
*
* RETURNS: The processor number for the CPU board.
*
* SEE ALSO: sysProcNumSet()
*/
int sysProcNumGet (void)
{
    return (sysProcNum);
}

/******************************************************************************
*
* sysProcNumSet - set the processor number
*
* This routine sets the processor number for the CPU board.  Processor numbers
* should be unique on a single backplane.
*
* For bus systems, it is assumes that processor 0 is the bus master and
* exports its memory to the bus.
*
* RETURNS: N/A
*
* SEE ALSO: sysProcNumGet()
*/
void sysProcNumSet
    (
    int procNum   /* processor number */
    )
{
    sysProcNum = procNum;
}


/*******************************************************************************
*
* sysUsDelay - Delay the specified amount of time (in microseconds)
*
* This routine will delay for at least the specified amount of time using the
* lower 32 bit "word" of the Time Base register as the timer.  The accuracy of
* the delay increases as the requested delay increases due to a certain amount
* of overhead. The overhead can be negated by using SYS_US_DELAY_BIAS defined
* in confiig.h. Calibrate to your system needs.
*
* Interrupt or task context switches may interfere with this routine and should
* be disabled if more accuracy is desired.
*
* NOTE: This routine will not relinquish the CPU. It is meant to perform a busy
* loop delay.
*
* RETURNS: N/A
*
*/
void sysUsDelay
    (
    UINT32 delay  /* Delay time in micro-seconds */
    )
{
    UINT32 baselineTickCount;
    UINT32 ticksToWait;
    UINT32 dummy;
    UINT32 currentTime;
    UINT32 oneUsDelay;
    UINT32 delta;

    /*
     * Get the Time Base Lower register tick count, this will be used
     * as the baseline.
     */
    vxTimeBaseGet(&dummy, &baselineTickCount);

    /* Account for function overhead */
#if (SYS_US_DELAY_BIAS < 0)
    if (delay > -SYS_US_DELAY_BIAS)
    {
        delay += SYS_US_DELAY_BIAS;
    }
#elif (SYS_US_DELAY_BIAS > 0)
    if (delay > SYS_US_DELAY_BIAS)
    {
        delay += SYS_US_DELAY_BIAS;
    }
#endif

    /* Exit if no delay count */
    if (!delay)
    {
        return;
    }

    /*
     * Calculate number of ticks equal to 1 microsecond
     *
     * The Time Base register ticks once per 4 system bus cycles:
     *
     * e.g., 66666666 cycles     1 tick      1 second             16.67 tick
     *       ---------------  *  ------   *  --------          =  ----------
     *       second              4 cycles    1000000 microsec     microsec
     */

    /*
     * Significant precision may be lost depending on the bus speed.
     * oneUsDelay will be represented as a fixed point number with it's
     * decimal point at the 100s digit. Example, if oneUsDelay physically
     * contains 16666, it's real value is 166.66. The decial will be moved
     * back to 1 when the final calculation is rescaled.
     */
    oneUsDelay = CORE_CLOCK_FREQ / 10000;

    /* Convert delay time into ticks and divide by 100 to rescale */
    ticksToWait = (oneUsDelay * delay) / 100;

    while(1)
    {
        /*
        ** subtracting an unsigned from a signed number is a faster way of
        ** calculating the delta while at the same time compensating for
        ** rollover
        */
        vxTimeBaseGet(&dummy, &currentTime);
        delta = currentTime - (int)baselineTickCount;
        if (delta >= ticksToWait) return;
    }
}

/*******************************************************************************
*
* sysMsDelay - Delay at least the specified amount of time (in milliseconds)
*
* This routine will delay for at least the specified amount of time using the
* lower 32 bit "word" of the Time Base register as the timer.
*
* Interrupt or task context switches may interfere with this routine and should
* be disabled if more accuracy is desired.
*
* NOTE: This routine will not relinquish the CPU. It is meant to perform a busy
* loop delay.
*
* RETURNS: N/A
*
*/
void sysMsDelay
    (
    UINT32 delay   /* Delay time in milli-seconds */
    )
{
    sysUsDelay(delay*1000);
}

/*******************************************************************************
*
* sysAssert - Display visual indications of an assert when taken.
*
* This routine can be used to trap asserts from the CSP.
*
* This routine when called with the file parameter equal to XNULL configures
* itself with the CSP to be invoked when CSP code detects an assert.
*
* RETURNS: N/A
*/
#ifndef NDEBUG
static void sysAssert
    (
    char* file,   /* File where assert occurred */
    int line      /* line within file where assert occurred */
    )
{
    static int configured = XFALSE;

    if ((file == XNULL) && (!configured))
    {
        XAssertSetCallback(sysAssert);
        configured = XTRUE;
        return;
    }

#ifdef INCLUDE_LOGGING
    if (file)
    {
        logMsg("\a\aassert %s line %d\n", (int)file, line, 3, 4, 5, 6);
    }
#endif
}

#endif  /* NDEBUG */


#ifdef INCLUDE_CSP_BOOT_ERROR_LOG
/******************************************************************************
*
* sysCspBootErrorLog - Log an error detected interacting with the CSP at boot
*
* The line number and file name are saved. Up to MAX_CSP_ERROR_RECORDS are
* saved. The info can be printed out by calling sysCspBootErrorLogShow().
*
*/
static void sysCspBootErrorLog
(
    int line,    /* Line number within file name */
    char* file   /* filename where error occurred */
)
{
    if (sysXInitErrorCtr >= MAX_CSP_BOOT_ERROR_RECORDS)
    {
        sysXInitErrorCtr++;
        return;
    }

    sysCspErrorLine[sysXInitErrorCtr] = line;
    sysCspErrorFile[sysXInitErrorCtr] = file;
    sysXInitErrorCtr++;
}

/******************************************************************************
*
* sysCspBootErrorLogShow - Display errors detected during boot time.
*
*/
void sysCspBootErrorLogShow(void)
{
    int i;

    if (sysXInitErrorCtr)
    {
        printf("%d CSP boot errors logged\n",sysXInitErrorCtr);

        for (i=0; i<sysXInitErrorCtr && i<MAX_CSP_BOOT_ERROR_RECORDS; i++)
        {
            printf("%4d : %s\n", sysCspErrorLine[i],sysCspErrorFile[i]);
        }
    }
}
#endif  /* INCLUDE_CSP_BOOT_ERROR_LOG */


/*
 * Include usrAppInit.c only if not using the Project Facility which
 * independently compiles its own usrAppInit.c
 */
#ifndef PRJ_BUILD
#  ifdef INCLUDE_USER_APPL
#    include "usrAppInit.c"
#  endif
#endif

