/* sysNet.c - System network hardware setup */

/*******************************************************************************
   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE.

   (c) Copyright 2003 Xilinx Inc.
   All rights reserved.
*******************************************************************************/

/* Copyright 1984-1997 Wind River Systems, Inc. */

/*
TODO - Start new revision history

modification history
--------------------
01a,02jul03,rpm  Added tags to integrate the XGemac adapter
01a,18feb03,rmm  Updated for VxWorks 5.5
01b,11may05,rmm  Added tags to integrate the XTemac adapter
*/

/*
DESCRIPTION
This library contains board-specific routines that coordinate the
initialization of system network hardware. This may include invoking the
END device drivers init routines, functions for getting/setting MAC
addresses, etc...

TODO - Update to suit your system's architecture.

This file contributes nothing if INCLUDE_NETWORK is not defined
*/

/* includes */

#include "vxLib.h"
#include "stdio.h"
#include "ctype.h"
#include "ppc405_0.h"

/***** GLOBALS *****/

#define MAX_END_LOAD_STR_SIZE       100

/*
 * A load string and MAC address buffer is declared for each END Ethernet
 * device.
 *
 * TODO: Defining the MAC addresses here in the BSP is done out of convenience.
 *       Update the MAC address values and implementation to suit your system's
 *       architecture.
 */

#ifdef INCLUDE_XEMAC_VXWORKS5_4



#endif

#ifdef INCLUDE_XGEMAC_VXWORKS5_4

#include "xgemac.h"



#endif

#ifdef INCLUDE_XEMACLITE_VXWORKS5_4



#include "xemaclite.h"

#endif

#ifdef INCLUDE_XTEMAC_VXWORKS5_4

#include "xtemac.h"

char XTemacLoadString0[MAX_END_LOAD_STR_SIZE];

static char XTemacMacAddr0[6] = { 0x6, 0x5, 0x4, 0x3, 0x2, 0x1 };

#endif

/* Forward declarations */
#ifdef INCLUDE_XEMAC_VXWORKS5_4
STATUS sysXEmacEndBldLoadStr(char *LoadStr, unsigned int DeviceId,
                             char *MacAddrBuffer, unsigned int IntrId);
#endif

#ifdef INCLUDE_XGEMAC_VXWORKS5_4
STATUS sysXGemacEndBldLoadStr(char *LoadStr, unsigned int DeviceId,
                              char *MacAddrBuffer, unsigned int IntrId);
#endif

#ifdef INCLUDE_XEMACLITE_VXWORKS5_4
STATUS sysXEmacLiteEndBldLoadStr(char *LoadStr, unsigned int DeviceId,
                                 char *MacAddrBuffer, unsigned int IntrId);
#endif

#ifdef INCLUDE_XTEMAC_VXWORKS5_4
STATUS sysXTemacEndBldLoadStr(char *LoadStr, unsigned int DeviceId,
                              char *MacAddrBuffer, unsigned int IntrId);
#endif

/******************************************************************************
*
* sysNetHwInit - initialize the network interface
*
* This routine initializes the network hardware to a quiescent state.  It
* does not connect interrupts.
*
* Only polled mode operation is possible after calling this routine.
* Interrupt mode operation is possible after the memory system has been
* initialized and sysNetHwInit2() has been called.
*
* RETURNS: N/A
*
* SEE ALSO: sysNetHwInit2()
*/

void sysNetHwInit(void)
{
    /*
     * Build the load string for the EMAC Ethernet core.
     */
#ifdef INCLUDE_XEMAC_VXWORKS5_4
#endif

#ifdef INCLUDE_XGEMAC_VXWORKS5_4
#endif

#ifdef INCLUDE_XEMACLITE_VXWORKS5_4
#endif

#ifdef INCLUDE_XTEMAC_VXWORKS5_4
    sysXTemacEndBldLoadStr(XTemacLoadString0, XPAR_TRIMODE_MAC_GMII_DEVICE_ID,
                           XTemacMacAddr0, XPAR_OPB_INTC_0_TRIMODE_MAC_GMII_IP2INTC_IRPT_INTR);
#endif

    return;
}

/******************************************************************************
*
* sysNetHwInit2 - initialize additional features of the network interface
*
* This routine completes initialization needed for interrupt mode operation
* of the network device drivers.  Interrupt handlers can be connected.
* Interrupt or DMA operations can begin.
*
* RETURNS: N/A
*
*/
void sysNetHwInit2(void)
{
    return;
}

#ifdef INCLUDE_XEMAC_VXWORKS5_4

/******************************************************************************
*
* sysXEmacEndBldLoadStr - builds the XEmacEndAdapter_Load parameter string
*
* RETURNS:
*
*/
STATUS sysXEmacEndBldLoadStr(char *LoadStr, unsigned int DeviceId,
                             char *MacAddrBuffer, unsigned int IntrId)
{

    /*
     * Build the initialization string.  It looks like this:
     *
     * <DeviceId>:<VersionId>:<MacAddrPtr>:
     * <BufferMemoryAddress>:<BufferMemorySize>:
     * <DescriptorMemoryAddress>:<DescriptorMemorySize>:
     * <CacheLineSize>:<IntrId>
     *
     * The VxWorks unit number will be tacked onto the beginning by
     * muxEndLoad
     */

    sprintf(LoadStr, "0x%x:1.00a:0x%x:0:0:0:0:%d:0x%x",
            DeviceId,
            (unsigned int)MacAddrBuffer,
            _CACHE_ALIGN_SIZE,
            IntrId);

    return OK;
}

#endif /* INCLUDE_XEMAC_VXWORKS5_4 */

#ifdef INCLUDE_XGEMAC_VXWORKS5_4

/******************************************************************************
*
* sysXGemacEndBldLoadStr - builds the XGemacEndAdapter_Load parameter string
*
* RETURNS:
*
*/
STATUS sysXGemacEndBldLoadStr(char *LoadStr, unsigned int DeviceId,
                              char *MacAddrBuffer, unsigned int IntrId)
{

    /*
     * Build the initialization string.  It looks like this:
     *
     * <DeviceId>:<VersionId>:<MacAddrPtr>:
     * <BufferMemoryAddress>:<BufferMemorySize>:
     * <DescriptorMemoryAddress>:<DescriptorMemorySize>:
     * <CacheLineSize>:<IntrId>:<InitialOptions>
     *
     * The VxWorks unit number will be tacked onto the beginning by
     * muxEndLoad
     */

    sprintf(LoadStr, "0x%x:1.00a:0x%x:0:0:0:0:%d:0x%x:0x%x",
            DeviceId,
            (unsigned int)MacAddrBuffer,
            _CACHE_ALIGN_SIZE,
            IntrId,
            (unsigned int)(XGE_AUTO_NEGOTIATE_OPTION |
                           XGE_FDUPLEX_OPTION |
                           XGE_INSERT_PAD_OPTION |
                           XGE_INSERT_FCS_OPTION |
                           XGE_FLOW_CONTROL_OPTION));

    return OK;
}

#endif /* INCLUDE_XGEMAC_VXWORKS5_4 */


#ifdef INCLUDE_XEMACLITE_VXWORKS5_4

/******************************************************************************
*
* sysXEmacLiteEndBldLoadStr - builds the XEmacLiteEndAdapter_Load parameter
* string
*
* RETURNS:
*
*/
STATUS sysXEmacLiteEndBldLoadStr(char *LoadStr, unsigned int DeviceId,
                                 char *MacAddrBuffer, unsigned int IntrId)
{

    /*
     * Build the initialization string.  It looks like this:
     *
     * <DeviceId>:<MacAddrPtr>:
     * <BufferMemoryAddress>:<BufferMemorySize>:
     * <CacheLineSize>:<IntrId>
     *
     * The VxWorks unit number will be tacked onto the beginning by
     * muxEndLoad
     */

    sprintf(LoadStr, "0x%x:0x%x:0:0:%d:0x%x",
            DeviceId,
            (unsigned int)MacAddrBuffer,
            _CACHE_ALIGN_SIZE,
            IntrId);

    return OK;
}

#endif /* INCLUDE_XEMACLITE_VXWORKS5_4 */


#ifdef INCLUDE_XTEMAC_VXWORKS5_4

/******************************************************************************
*
* sysXTemacEndBldLoadStr - builds the XTemacEndAdapter_Load parameter string
*
* RETURNS:
*
*/
STATUS sysXTemacEndBldLoadStr(char *LoadStr, unsigned int DeviceId,
                              char *MacAddrBuffer, unsigned int IntrId)
{

    /*
     * Build the initialization string.  It looks like this:
     *
     * <DeviceId>:<MacAddrPtr>:
     * <BufferMemoryAddress>:<BufferMemorySize>:
     * <DescriptorMemoryAddress>:<DescriptorMemorySize>:
     * <RxBdCnt>:<TxBdCnt>:
     * <CacheLineSize>:<IntrId>:<MTU>
     *
     * The VxWorks unit number will be tacked onto the beginning by
     * muxEndLoad.
     *
     * See xtemac_end_adapter.c for detailed information about these fields.
     *
     */

    sprintf(LoadStr, "%d:0x%x:0x%x:0x%x:0x%x:0x%x:%d:%d:%d:%d:%d",
            DeviceId,
            (unsigned int)MacAddrBuffer,
            0, 0,   /* buffer base addr, buffer space length in bytes */
            0, 0,   /* descriptor base addr, descriptor space length in bytes */
            0, 0,   /* rxbd count, txbd count */
            _CACHE_ALIGN_SIZE,
            IntrId,
            1500);

    return OK;
}

#endif /* INCLUDE_XTEMAC_VXWORKS5_4 */


