/* sysPlbOpb.c - System PLB/OPB bridge & arbiter library */

/*******************************************************************************
   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE.

   (c) Copyright 2003 Xilinx Inc.
   All rights reserved.
*******************************************************************************/

/*
  
TODO - Start a new revision history
  
modification history
--------------------
01a,18feb03,rmm  First release
                 
*/

/*
DESCRIPTION
This library contains board-specific routines for coordination and
initialization of PLB and OPB related components. 

*/

#include "vxWorks.h"
#include "xparameters.h"

/* Include header file for each component */
#ifdef INCLUDE_XPLB_ARBITER
#  include "xplbarb_l.h"
#  include "xplbarb.h"
#endif

#ifdef INCLUDE_XOPB_ARB
#  include "xopbarb_l.h"
#  include "xopbarb.h"
#endif

#ifdef INCLUDE_XOPB_TO_PLB
#  include "xopb2plb_l.h"
#  include "xopb2plb.h"
#endif

#ifdef INCLUDE_XPLB_TO_OPB
#  include "xplb2opb_l.h"
#  include "xplb2opb.h"
#endif

/* Global data */

/* TODO - define storage for any PLB/OPB component instances */
#if 0
#ifdef INCLUDE_XPLB_ARBITER
XPlbArb sysXPlbArb;
#endif

#ifdef INCLUDE_XOPB_ARB
XOpbArb sysXOpbArb;
#endif

#ifdef INCLUDE_XOPB_TO_PLB
XOpb2Plb sysXOpb2Plb;
#endif

#ifdef INCLUDE_XPLB_TO_OPB
XPlb2Opb sysXPlb2Opb;
#endif

#endif /* if 0 */

/* Forward declarations */

/******************************************************************************
*
* sysPlbOpbInit - Perform first level initalization of PLB & OPB arbiters and
* bridges
* 
* This function is called from sysHwInit() with interrupts locked.
*
* RETURNS: OK if successful, ERROR otherwise
*
* SEE ALSO: sysHwInit()
*/
static STATUS sysPlbOpbInit(void)
{

    /* TODO - initialize any PLB/OPB instance here, see examples below */    
#if 0    
#ifdef INCLUDE_XPLB_ARBITER
    if (XPlbArb_Initialize(&sysXPlbArb, SYS_PLBARB_DEV_ID) != XST_SUCCESS)
    {
        BOOT_LOG_ERROR();
        return(ERROR);
    }
#endif

#ifdef INCLUDE_XOPB_ARB
    if (XOpbArb_Initialize(&sysXOpbArb, SYS_OPBARB_DEV_ID) != XST_SUCCESS)
    {
        BOOT_LOG_ERROR();
        return(ERROR);
    }
#endif
    
#ifdef INCLUDE_XPLB_TO_OPB
    if (XPlb2Opb_Initialize(&sysXPlb2Opb, SYS_PLB2OPB_BRIDGE_DEV_ID) != XST_SUCCESS)
    {
        BOOT_LOG_ERROR();
        return(ERROR);
    }
#endif

#ifdef INCLUDE_XOPB_TO_PLB
    if (XOpb2Plb_Initialize(&sysXOpb2Plb, SYS_OPB2PLB_BRIDGE_DEV_ID) != XST_SUCCESS)
    {
        BOOT_LOG_ERROR();
        return(ERROR);
    }
#endif

#endif /* if 0 */    
    return(OK);
}


/******************************************************************************
*
* sysPlbOpbInit2 - Perform second level initalization of PLB & OPB arbiters and
* bridges
* 
* This function is called from sysHwInit2() with interrupts enabled.
*
* RETURNS: OK if successful, ERROR otherwise
*
* SEE ALSO: sysHwInit()
*/
static STATUS sysPlbOpbInit2(void)
{
    /*
     * Enable the device's interrupt output signal to the critical interrupt
     * controller. For an interrupt to occur, you still need to call
     * sysCritIntEnable()
     */

    /* TODO - 2nd level initialization here here, see examples below */    
#if 0    

#ifdef INCLUDE_XPLB_ARBITER
    XPlbArb_EnableInterrupt(&sysXPlbArb);
#endif

#ifdef INCLUDE_XOPB_ARB
    /* Nothing to do */
#endif
    
#ifdef INCLUDE_XPLB_TO_OPB
    XPlb2Opb_EnableInterrupt(&sysXPlb2Opb);
#endif

#ifdef INCLUDE_XOPB_TO_PLB
    /* Nothing to do */
#endif

#endif /* if 0 */    
    return(OK);
}
