/* sysSerial.c - Serial device initialization */

/*******************************************************************************
   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE.

   (c) Copyright 2003 Xilinx Inc.
   All rights reserved.
*******************************************************************************/

/* Copyright 1984-1997 Wind River Systems, Inc. */

/*
TODO - Start new revision history

modification history
--------------------
01a,18feb03,rmm  Ported from VxWorks 5.4 template
*/

/*
DESCRIPTION
This library contains board-specific routines to coordinate the initialization
of system serial device hardware.

TODO - Update to suite your system's architecture.
*/

/*
 * Include SIO driver header file for each type of uart in the system
 */
#ifdef INCLUDE_XUARTNS550_VXWORKS5_4
#  include "xuartns550_adapter.h"
#endif
#ifdef INCLUDE_XUARTLITE_VXWORKS5_4
#  include "xuartlite_sio_adapter.h"
#endif

/*
 * Allocate serial channel structure for each uart in the system
 */
#ifdef INCLUDE_XUARTNS550_VXWORKS5_4
static XUartNs550SioChannel UartNs550SioChan[1];
#endif
#ifdef INCLUDE_XUARTLITE_VXWORKS5_4
static XUartLiteSioChannel UartLiteSioChan[0];
#endif

/*
 * Using the serial channel structures defined above, setup the SIO lookup
 * table. This table effectively maps an SIO channel to a specific ty
 */
static SIO_CHAN *sysSioChans[] =
{
#ifdef INCLUDE_XUARTNS550_VXWORKS5_4
    (SIO_CHAN *)&UartNs550SioChan[0],
#endif

#ifdef INCLUDE_XUARTLITE_VXWORKS5_4
#endif

#if !defined(INCLUDE_XUARTNS550_VXWORKS5_4) && !defined(INCLUDE_XUARTLITE_VXWORKS5_4)
    (SIO_CHAN *)ERROR
#endif
};

/******************************************************************************
*
* sysSerialHwInit - initialize the BSP serial devices to a quiescent state
*
* This routine initializes the BSP serial device descriptors and puts the
* devices in a quiescent state.  It is called from sysHwInit() with
* interrupts locked.
*
* RETURNS: N/A
*
* SEE ALSO: sysHwInit()
*/
void sysSerialHwInit(void)
{
#ifdef INCLUDE_XUARTNS550_VXWORKS5_4
     if (XUartNs550Adapter_DevInit(&UartNs550SioChan[0],
                                   XPAR_RS232_UART_2_DEVICE_ID, XPAR_OPB_INTC_0_RS232_UART_2_IP2INTC_IRPT_INTR) != XST_SUCCESS)
     {
         BOOT_LOG_ERROR();
     }
#endif
#ifdef INCLUDE_XUARTLITE_VXWORKS5_4
#endif
}

/******************************************************************************
*
* sysSerialHwInit2 - connect BSP serial device interrupts
*
* This routine connects the BSP serial device interrupts.  It is called from
* sysHwInit2().
*
* Serial device interrupts cannot be connected in sysSerialHwInit() because
* the kernel memory allocator is not initialized at that point, and
* intConnect() calls malloc().
*
* RETURNS: N/A
*
* SEE ALSO: sysHwInit2()
*/
void sysSerialHwInit2(void)
{
    int intLevel;

    intLevel = intLock();

#ifdef INCLUDE_XUARTNS550_VXWORKS5_4
    if (XUartNs550Adapter_DevInit2(
            &UartNs550SioChan[0]) != XST_SUCCESS)
    {
        BOOT_LOG_ERROR();
    }
#endif

#ifdef INCLUDE_XUARTLITE_VXWORKS5_4
#endif

    intUnlock(intLevel);
}

/******************************************************************************
*
* sysSerialChanGet - get the SIO_CHAN device associated with a serial channel
*
* This routine returns a pointer to the SIO_CHAN device associated
* with a specified serial channel.  It is called by usrRoot() to obtain
* pointers when creating the system serial devices, `/tyCo/x'.  It
* is also used by the WDB agent to locate its serial channel.
*
* RETURNS: A pointer to the SIO_CHAN structure for the channel, or ERROR
* if the channel is invalid.
*
*/
SIO_CHAN * sysSerialChanGet(int channel)
{
    /*
     * Verify channel is not out of bounds
     * NELEMENTS is defined in vxWorks.h
     */
    if (channel > NELEMENTS(sysSioChans))
    {
        return ((SIO_CHAN*)ERROR);
    }

    /*
     * Return SIO channel
     */
    return sysSioChans[channel];
}
